@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2013-2015 Brazil
+  Copyright(C) 2013-2016 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -34,10 +34,12 @@
 # include "mrb/mrb_query_logger.h"
 # include "mrb/mrb_void.h"
 # include "mrb/mrb_bulk.h"
+# include "mrb/mrb_pointer.h"
 # include "mrb/mrb_cache.h"
 # include "mrb/mrb_object.h"
 # include "mrb/mrb_object_flags.h"
 # include "mrb/mrb_database.h"
+# include "mrb/mrb_indexable.h"
 # include "mrb/mrb_table.h"
 # include "mrb/mrb_array.h"
 # include "mrb/mrb_hash_table.h"
@@ -63,6 +65,7 @@
 # include "mrb/mrb_table_cursor_flags.h"
 # include "mrb/mrb_content_type.h"
 # include "mrb/mrb_writer.h"
+# include "mrb/mrb_config.h"
 
 # include <mruby/array.h>
 # include <mruby/string.h>
@@ -144,10 +147,12 @@ mrb_groonga_init(mrb_state *mrb, mrb_value self)
   grn_mrb_query_logger_init(ctx);
   grn_mrb_void_init(ctx);
   grn_mrb_bulk_init(ctx);
+  grn_mrb_pointer_init(ctx);
   grn_mrb_cache_init(ctx);
   grn_mrb_object_init(ctx);
   grn_mrb_object_flags_init(ctx);
   grn_mrb_database_init(ctx);
+  grn_mrb_indexable_init(ctx);
   grn_mrb_table_init(ctx);
   grn_mrb_array_init(ctx);
   grn_mrb_hash_table_init(ctx);
@@ -173,6 +178,7 @@ mrb_groonga_init(mrb_state *mrb, mrb_value self)
   grn_mrb_table_cursor_flags_init(ctx);
   grn_mrb_content_type_init(ctx);
   grn_mrb_writer_init(ctx);
+  grn_mrb_config_init(ctx);
 
   grn_mrb_load(ctx, "initialize/post.rb");
 
@@ -191,6 +197,25 @@ grn_ctx_impl_mrb_init_bindings(grn_ctx *ctx)
   mrb_funcall(mrb, mrb_obj_value(ctx->impl->mrb.module), "init", 0);
 }
 
+static void *
+grn_ctx_impl_mrb_allocf(mrb_state *mrb, void *ptr, size_t size, void *ud)
+{
+  grn_ctx *ctx = ud;
+
+  if (size == 0) {
+    if (ptr) {
+      grn_free(ctx, ptr, __FILE__, __LINE__, __FUNCTION__);
+    }
+    return NULL;
+  } else {
+    if (ptr) {
+      return grn_realloc(ctx, ptr, size, __FILE__, __LINE__, __FUNCTION__);
+    } else {
+      return grn_malloc(ctx, size, __FILE__, __LINE__, __FUNCTION__);
+    }
+  }
+}
+
 void
 grn_ctx_impl_mrb_init(grn_ctx *ctx)
 {
@@ -205,7 +230,7 @@ grn_ctx_impl_mrb_init(grn_ctx *ctx)
     ctx->impl->mrb.groonga.operator_class = NULL;
   } else {
     mrb_state *mrb;
-    mrb = mrb_open();
+    mrb = mrb_open_allocf(grn_ctx_impl_mrb_allocf, ctx);
     ctx->impl->mrb.state = mrb;
     ctx->impl->mrb.base_directory[0] = '\0';
     grn_ctx_impl_mrb_init_bindings(ctx);
